@@ -178,7 +178,8 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         # # directly (but complain about it).
         if self.global_options.hex_entropy_score:
             warnings.warn(
-                "--hex-entropy-score is deprecated. Use --entropy-sensitivity instead.",
+                "--hex-entropy-score is deprecated and will be removed in tartufo 4.x. "
+                "Please use --entropy-sensitivity instead.",
                 DeprecationWarning,
             )
             return self.global_options.hex_entropy_score
@@ -196,7 +197,8 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         # # directly (but complain about it).
         if self.global_options.b64_entropy_score:
             warnings.warn(
-                "--b64-entropy-score is deprecated. Use --entropy-sensitivity instead.",
+                "--b64-entropy-score is deprecated and will be removed in tartufo 4.x. "
+                "Please use --entropy-sensitivity instead.",
                 DeprecationWarning,
             )
             return self.global_options.b64_entropy_score
@@ -267,7 +269,7 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
             if deprecated:
                 warnings.warn(
                     "Old format of --include-path-patterns option and config file setup include-path-patterns "
-                    "= ['inclusion pattern'] has been deprecated and will be removed in a future version. "
+                    "= ['inclusion pattern'] has been deprecated and will be removed in tartufo 4.x. "
                     "Make sure all the inclusions are set up using new pattern i.e. include-path-patterns = "
                     "[{path-pattern='inclusion pattern',reason='reason for inclusion'}] in the config file",
                     DeprecationWarning,
@@ -316,7 +318,7 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
             if deprecated:
                 warnings.warn(
                     "Old format of --exclude-path-patterns option and config file setup exclude-path-patterns "
-                    "= ['exclusion pattern'] has been deprecated and will be removed in a future version. "
+                    "= ['exclusion pattern'] has been deprecated and will be removed in tartufo 4.x. "
                     "Make sure all the exclusions are set up using new pattern i.e. exclude-path-patterns = "
                     "[{path-pattern='exclusion pattern',reason='reason for exclusion'}] in the config file",
                     DeprecationWarning,
@@ -404,7 +406,7 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
             if deprecated:
                 warnings.warn(
                     "Configuring exclude-signatures as string has been deprecated and support for this format will "
-                    "be removed in the future. Please make sure to update your exclude-signatures configuration to "
+                    "be removed in tartufo 4.x. Please update your exclude-signatures configuration to "
                     "an array of tables. For example: exclude-signatures = [{signature='signature', reason='The "
                     "reason of excluding the signature'}]",
                     DeprecationWarning,
@@ -591,16 +593,27 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         :param chunk: The chunk of data to be scanned
         """
 
-        for line in chunk.contents.split("\n"):
-            for word in line.split():
+        for line in (x for x in chunk.contents.split("\n") if x):
+            # If the chunk is diff output, the first character of each line is
+            # generated metadata ("+", "-", etc.) that is not part of actual
+            # repository content, and it should be ignored.
+            extra_char: Optional[str]
+            if chunk.is_diff:
+                extra_char = line[0]
+                analyze = line[1:]
+            else:
+                extra_char = None
+                analyze = line
+            for word in analyze.split():
                 for string in util.find_strings_by_regex(word, BASE64_REGEX):
                     yield from self.evaluate_entropy_string(
-                        chunk, line, string, self.b64_entropy_limit
+                        chunk, analyze, string, self.b64_entropy_limit, extra_char
                     )
                 for string in util.find_strings_by_regex(word, HEX_REGEX):
                     yield from self.evaluate_entropy_string(
-                        chunk, line, string, self.hex_entropy_limit
+                        chunk, analyze, string, self.hex_entropy_limit, extra_char
                     )
+                extra_char = None
 
     def evaluate_entropy_string(
         self,
@@ -608,6 +621,7 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         line: str,
         string: str,
         min_entropy_score: float,
+        backwards_compatibility_prefix: Optional[str],
     ) -> Generator[Issue, None, None]:
         """Check entropy string using entropy characters and score.
 
@@ -615,13 +629,42 @@ class ScannerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         :param line: Source line containing string of interest
         :param string: String to check
         :param min_entropy_score: Minimum entropy score to flag
+        :param backwards_compatibility_prefix: Possible prefix character
         :return: Generator of issues flagged
+
+        If the string in "string" would result in an Issue (i.e. it has high
+        entropy and is not excluded), and backwards_compatibility_prefix is not
+        None, re-check for exclusions based on "prefix" + "string". This preserves
+        the utility of signatures generated by earlier tartufo versions which did
+        not handle "diff" chunks correctly.
         """
+
         if not self.signature_is_excluded(string, chunk.file_path):
             entropy_score = self.calculate_entropy(string)
             if entropy_score > min_entropy_score:
                 if self.entropy_string_is_excluded(string, line, chunk.file_path):
                     self.logger.debug("line containing entropy was excluded: %s", line)
+                elif (
+                    backwards_compatibility_prefix is not None
+                    and self.signature_is_excluded(
+                        backwards_compatibility_prefix + string, chunk.file_path
+                    )
+                ):
+                    self.logger.debug(
+                        "line containing entropy was excluded (old signature): %s", line
+                    )
+                    # We should tell the user to update their old signature
+                    new_signature = util.generate_signature(string, chunk.file_path)
+                    old_signature = util.generate_signature(
+                        backwards_compatibility_prefix + string, chunk.file_path
+                    )
+                    warnings.warn(
+                        f"Signature {old_signature} was generated by an old version of tartufo and is deprecated. "
+                        "tartufo 4.x will not recognize this signature. "
+                        f"Please update your configuration to use signature {new_signature} instead.",
+                        DeprecationWarning,
+                    )
+
                 else:
                     yield Issue(types.IssueType.Entropy, string, chunk)
 
@@ -864,6 +907,7 @@ class GitRepoScanner(GitScanner):
                         blob,
                         file_path,
                         util.extract_commit_metadata(curr_commit, branch_name),
+                        True,
                     )
 
             # Finally, yield the first commit to the branch
@@ -876,6 +920,7 @@ class GitRepoScanner(GitScanner):
                         blob,
                         file_path,
                         util.extract_commit_metadata(curr_commit, branch_name),
+                        True,
                     )
 
 
@@ -919,7 +964,7 @@ class GitPreCommitScanner(GitScanner):
             | pygit2.GIT_DIFF_SHOW_UNTRACKED_CONTENT,
         )
         for blob, file_path in self._iter_diff_index(diff_index):
-            yield types.Chunk(blob, file_path, {})
+            yield types.Chunk(blob, file_path, {}, True)
 
 
 class FolderScanner(ScannerBase):
@@ -946,7 +991,7 @@ class FolderScanner(ScannerBase):
         """Yield the individual files in the target directory."""
 
         for blob, file_path in self._iter_folder():
-            yield types.Chunk(blob, file_path, {})
+            yield types.Chunk(blob, file_path, {}, False)
 
     def _iter_folder(self) -> Generator[Tuple[str, str], None, None]:
         folder_path = pathlib.Path(self.target)
